using System;
using System.Data;
using System.Data.SqlClient;

namespace MessageBoard
{
	/// <summary>
	/// Summary description for Category.
	/// </summary>
	public class Category : Connection
	{
		#region Fields
		private int _Id;
		private Group _Group;
		private string _Title;
		private string _Description;
		private int _SortingOrder;
		private Boolean _Active;
		private User _AddedBy;
		private DateTime _AddedOn;
		#endregion

		#region Constructors
		public Category()
		{
			_Title = "";
			_Description = "";
			_SortingOrder = 1;
			_Active = true;
		}
		#endregion

		#region Functions
		/// <summary>
		/// Fill the category properties
		/// </summary>
		/// <param name="dr"></param>
		/// <returns></returns>
		public static Category Get(System.Data.DataRow dr)
		{
			Category category = new Category();
			category.Id = int.Parse(dr["Id"].ToString());
			category.Group = Group.Get(int.Parse(dr["GroupId"].ToString()));
			category.Title = dr["Title"].ToString();
			category.Description = dr["Description"].ToString();
			category.SortingOrder = int.Parse(dr["SortingOrder"].ToString());
			category.Active = Convert.ToBoolean(dr["Active"].ToString());
			category.AddedBy = User.Get(int.Parse(dr["AddedBy"].ToString()));
			category.AddedOn = Convert.ToDateTime(dr["AddedOn"].ToString());
			return category;
		}

		/// <summary>
		/// Get a category by its Id
		/// </summary>
		/// <param name="id"></param>
		/// <returns></returns>
		public static Category Get(int id)
		{
			DataSet ds = new DataSet();
			SqlDataAdapter getData = new SqlDataAdapter("SELECT * FROM " + Connection.tblCategory + " WHERE [Id] = @Id", DbConnection);
			getData.SelectCommand.Parameters.Add("@Id", id);
			getData.Fill(ds);
			if(ds.Tables[0].Rows.Count>0)
			{
				return Get(ds.Tables[0].Rows[0]);
			}
			else
			{
				return null;
			}
		}

		/// <summary>
		/// Get all the categories in a given group
		/// </summary>
		/// <param name="group"></param>
		/// <returns></returns>
		public static System.Collections.ArrayList GetAll(Group group)
		{
			System.Collections.ArrayList arrayList = new System.Collections.ArrayList();
			DataSet ds = new DataSet();
			SqlDataAdapter getData = new SqlDataAdapter("SELECT * FROM " + Connection.tblCategory + " WHERE [GroupId] = @GroupId", DbConnection);
			getData.SelectCommand.Parameters.Add("@GroupId", group.Id);
			getData.Fill(ds);
			foreach(System.Data.DataRow dr in ds.Tables[0].Rows)
			{
				arrayList.Add(Get(dr));
			}
			return arrayList;
		}		
		#endregion

		#region Methods
		public void Insert()
		{
			DataSet ds = new DataSet();
			SqlDataAdapter getData = new SqlDataAdapter(@"
			INSERT INTO " + Connection.tblCategory + @" 
			([GroupId], [Title], [Description], [SortingOrder], [Active], [AddedBy], [AddedOn])
			VALUES
			(@GroupId, @Title, @Description, @SortingOrder, @Active, @AddedBy, @AddedOn);
			SELECT @@identity as i", DbConnection);
			getData.SelectCommand.Parameters.Add("@GroupId", Group.Id);
			getData.SelectCommand.Parameters.Add("@Title", Title);
			getData.SelectCommand.Parameters.Add("@Description", Description);
			getData.SelectCommand.Parameters.Add("@SortingOrder", SortingOrder);
			getData.SelectCommand.Parameters.Add("@Active", Active);
			getData.SelectCommand.Parameters.Add("@AddedBy", _AddedBy.Id);
			getData.SelectCommand.Parameters.Add("@AddedOn", _AddedOn);
			
			getData.Fill(ds);
			if(ds.Tables[0].Rows.Count>0)
			{
				Id = int.Parse(ds.Tables[0].Rows[0]["i"].ToString());
			}
		}

		public void Update()
		{
			DataSet ds = new DataSet();
			SqlDataAdapter getData = new SqlDataAdapter(@"
			UPDATE " + Connection.tblCategory + @" SET 
			[GroupId] = @GroupId,
			[Title] = @Title,
			[Description] = @Description,
			[SortingOrder] = @SortingOrder,
			[Active] = @Active,
			[AddedBy] = @AddedBy,
			[AddedOn] = @AddedOn
			WHERE [Id] = @Id", DbConnection);
			getData.SelectCommand.Parameters.Add("@GroupId", Group.Id);
			getData.SelectCommand.Parameters.Add("@Title", Title);
			getData.SelectCommand.Parameters.Add("@Description", Description);
			getData.SelectCommand.Parameters.Add("@SortingOrder", SortingOrder);
			getData.SelectCommand.Parameters.Add("@Active", Active);
			getData.SelectCommand.Parameters.Add("@AddedBy", _AddedBy.Id);
			getData.SelectCommand.Parameters.Add("@AddedOn", _AddedOn);
			
			getData.SelectCommand.Parameters.Add("@Id", Id);
			getData.Fill(ds);
		}

		public void Delete()
		{
			// Delete the related forums first.
			DeleteForums();

			DataSet ds = new DataSet();
			SqlDataAdapter getData = new SqlDataAdapter(@"
			DELETE FROM " + Connection.tblCategory + @" WHERE [Id] = @Id", DbConnection);
			getData.SelectCommand.Parameters.Add("@Id", Id);
			getData.Fill(ds);
		}

		/// <summary>
		/// Delete the related forums.
		/// </summary>
		private void DeleteForums()
		{
			foreach(Forum forum in Forum.GetAll(this))
			{
				forum.Delete();
			}
		}
		#endregion

		#region Properties
		
		/// <summary>
		/// The Id of the category.
		/// </summary>
		public int Id
		{
			get
			{
				return _Id;
			}
			set
			{
				_Id = value;
			}
		}

		/// <summary>
		/// The group of the category.
		/// </summary>
		public Group Group
		{
			get
			{
				return _Group;
			}
			set
			{
				_Group = value;
			}
		}

		/// <summary>
		/// The Title of the category.
		/// </summary>
		public string Title
		{
			get
			{
				return _Title;
			}
			set
			{
				_Title = value;
			}
		}

		/// <summary>
		/// the Description of the category.
		/// </summary>
		public string Description
		{
			get
			{
				return _Description;
			}
			set
			{
				_Description = value;
			}
		}

		/// <summary>
		/// The Sorting Order of the category.
		/// </summary>
		public int SortingOrder
		{
			get
			{
				return _SortingOrder;
			}
			set
			{
				_SortingOrder = value;
			}
		}

		/// <summary>
		/// If this category is active or not.
		/// </summary>
		public Boolean Active
		{
			get
			{
				return _Active;
			}
			set
			{
				_Active = value;
			}
		}

		/// <summary>
		/// The user who add this category.
		/// </summary>
		public User AddedBy
		{
			get
			{
				return _AddedBy;
			}
			set
			{
				_AddedBy = value;
			}
		}

		/// <summary>
		/// The date this category is added.
		/// </summary>
		public DateTime AddedOn
		{
			get
			{
				return _AddedOn;
			}
			set
			{
				_AddedOn = value;
			}
		}	
		#endregion
	}
}
